# 1. Encodes high-functionality python SDK for streaming data from senspots into .txt, .csv, .xml, etc.
# 2. Streams data from USB sink serial port to a local sqlite db
# 3. ETL functions to push locally streamed data to MySQL db on resensys.net
# 4. Option to open a streaming window to view all present streaming devices




import mysql.connector
from senstream.resensys import Resensys
from datetime import datetime,timedelta
import pandas as pd

# 1. Handles all operations for remote configurations
# 2. Handles setting name of a site in a user account
# 3. Used to extract information about the health of the site (e.g. activity status, low battery, no charging, etc.)

class SeniMax(Resensys):
    def __init__(self):
        return
    
    # get the name of the site
    def siteName(self):
        return

    # set the site name to something else
    def siteName(self,name):
        return
    
    # return a summary of the status of the device
    def status(self):
        return

    def txInterval(self):
        return


# 1. User can define custom scripts and operations on given senspot timestreams
# 2. Ability to set alert thresholds for user defined senstreams
# 3. Connects to the resensys alert_manager to trigger on given condition
# 4. Ability to add the series to resensys.net as a custom timestream


import mysql.connector
from senstream.resensys import Resensys
from senstream.senspot import SenSpot
import pandas as pd

# 1. Handles alert management configurations
# 2. Change the activity status of a user of the alert service
# 3. Pull the current static alert levels for a given device
# 4. Set new static alert levels per device-quantity pair
# 5. Defines an "alert trigger" which automatically sends an alert when a user-defined condition is met
# 6. Specify who will be recieving alerts (emails and mobile numbers)
# 7. Sync alerts across systems (sites, senscope, and webportal)

class AlertManager(Resensys):
    def __init__(self,parent):
        super().__init__(parent.username, parent.password)
        self.conn = parent.conn

    # check the current alert levels for all devices
    def alerts(self,senspot=None,format="json"):
        
        cursor = self.conn.cursor()
        
        if senspot is not None:
            # parse the senspot object
            query = f"SELECT DeviceID1 as DID,SiteID1 as SID,Quantity1 as DataFormat,Quantity1Name as QuantityName,Offset1 as Offset,Coef1 as Coeff,ALARM_LL_1 as AlarmLow,WARNING_LL_1 as WarningLow,WARNING_HH_1 as WarningHigh,ALARM_HH_1 as AlarmHigh FROM {self.username}.AlertGen WHERE DeviceID1 = '{senspot.did}';"
            cursor.execute(query)
            senspot_alerts = cursor.fetchall()
            field_names = [i[0] for i in cursor.description]

            if senspot_alerts is not None:
                if format == "json":
                    alerts_dict = {}
                    for count,result in enumerate(alert_levels):
                        alerts_dict[str(count)] = {field_names[i]:result[i] for i in range(len(field_names))}
                    return alerts_dict
                elif format == "dataframe":
                    return pd.DataFrame(senspot_alerts,columns=field_names)
            else:
                print("Account not registered alert service user.")

        else:
            query = f"SELECT DeviceID1 as DID,SiteID1 as SID,Quantity1 as DataFormat,Quantity1Name as QuantityName,Offset1 as Offset,Coef1 as Coeff,ALARM_LL_1 as AlarmLow,WARNING_LL_1 as WarningLow,WARNING_HH_1 as WarningHigh,ALARM_HH_1 as AlarmHigh FROM {self.username}.AlertGen;"
            cursor.execute(query)
            alert_levels = cursor.fetchall()
            field_names = [i[0] for i in cursor.description]

            if alert_levels is not None:
                if format == "json":
                    alerts_dict = {}
                    for count,result in enumerate(alert_levels):
                        alerts_dict[str(count)] = {field_names[i]:result[i] for i in range(len(field_names))}
                    return alerts_dict
                elif format == "dataframe":
                    return pd.DataFrame(alert_levels,columns=field_names)
            else:
                print("Account not registered alert service user.")
    
    # handle registration of a user for the alert service
    def register(self):
        # check to see if the user is alread registered in active users
        cnx = mysql.connector.connect(user='tom',password='Manhattan44joke#',
                            host='resensys.net',use_pure=True)
        c = cnx.cursor()
        # define the query to the db
        query = f"SELECT AlertActivated FROM admin.ActiveUsers WHERE Username = '{self.username}';"
        c.execute(query)
        alerts_active = c.fetchall()[0][0]

        if alerts_active == 2:
            print("Account is already registered in the alert management system.")
        else:
            # add new user to admin.activeusers
            insert_new_alert_user = f"INSERT INTO `admin`.`ActiveUsers` (`Username`, `Password`, `DBName`, `AlertActivated`, `DBUID`, `DBPSSWD`, `UserCreationTime`) VALUES ('{self.username}', '{self.password}', '{self.username}', '2', '{self.password}', '{self.encoded_password}', '2020-11-20 12:00:00');"
            c.execute(insert_new_alert_user)
            cnx.commit()
            # then perform an insert_alert and update_quantitylist stored proceedures
        cnx.close()

    # handle the unregstering the user in the alert service
    def unregister(self):
        cnx = mysql.connector.connect(user='tom',password='Manhattan44joke#',
                            host='resensys.net',use_pure=True)
        c = cnx.cursor()
        remove_alert_user = f"DELETE FROM `admin`.`ActiveUsers` WHERE (`Username` = '{self.username}');"
        c.execute(remove_alert_user)
        cnx.commit()

    def updateAlert(self,dids,threshold_dict={}):
        return
        # update_command = UPDATE `Constellation`.`AlertGen` SET `WARNING_LL_1` = '7' WHERE (`idAlertGen` = '6188');

    # input an email address as a string to be added to database
    def addEmail(self,email):
        return
    
    # input a list of email addresses
    def updateEmail(self,email_list):
        return

    # input mobile number for recieving text message alerts
    def addMobile(self, mobile):
        return
    
    # input mobile numbers to recieve multiple messages at once
    def updateMobile(self,mobile_list):
        return

    



import re
import pandas as pd
from datetime import datetime
from datetime import timedelta

SenScopeConfiguration = {
    'MAX_NUM_ACTIVE_USERS':  20,
    'MAX_ALERT_NUM':    1000,
    'MAX_MIN_ALERT':    0,
    'MAX_DEVICE_BUFFER_SIZE':  1000,
    'MAX_DATAPOINT_PER_SERIE': 4096,

    'COLOR_CODE_WHITE': 0,
    'COLOR_CODE_YELLOW':   1,
    'COLOR_CODE_RED': 2,


    'MAX_VECTOR_LEN':   10000,
    'DATA_FORMAT_TEMP': 3002,
    'DATA_FORMAT_VLT':  3003,
    'DATA_FORMAT_TILT_X':  2001,
    'DATA_FORMAT_TILT_Y':  2002,
    'DATA_FORMAT_TILT_Z':  2003,
    'DATA_FORMAT_STRAIN':  1,

    'DUR_1_HOUR':  1,
    'DUR_2_HOUR':  2,
    'DUR_6_HOUR':  6,
    'DUR_12_HOUR':  12,
    'STANDARD_1_HOUR': 3600,
    'NUM_HOUR_IN_A_DAY':  24,

    'DF_SENSOR_STRAIN_LOW_RATE': 1,
    'DF_SENSOR_TILT_XYZ': 2,
    'DF_SENSOR_TILT_X': 2001,
    'DF_SENSOR_TILT_Y': 2002,
    'DF_SENSOR_TILT_Z': 2003,
    'DF_SENSOR_VOLT_TEMP': 3,
    'DF_SENSOR_LQI': 3001,
    'DF_SENSOR_TEMP': 3002,
    'DF_SENSOR_VOLT': 3003,
    'DF_SENSOR_DISPLACEMENT_LOW_RATE': 5,
    'DF_SENSOR_DISPLACEMENT_LOW_RATE_1inch': 5001,
    'DF_SENSOR_DISPLACEMENT_LOW_RATE_2inch': 5002,
    'DF_SENSOR_DISPLACEMENT_LOW_RATE_4inch': 5003,
    'DF_SENSOR_MOISTURE_TEMP_PROBE': 6,
    'DF_SENSOR_MOISTURE_PROBE_LOW_RATE': 6001,
    'DF_SENSOR_TEMP_PROBE_LOW_RATE': 6002,
    'DF_E_TILT_Y_FORMAT': 7,
    'DF_DOUBLE_TEMP_INFO': 8,
    'DF_SENSOR_PADDING': 15,
    'DF_SENSOR_DOUBLE_TEMP_INFO1': 8001,
    'DF_SENSOR_DOUBLE_TEMP_INFO2': 8002,
    'DF_SENSOR_ACCELERATION_XYZ': 10,
    'DF_SENSOR_ACCELERATION_X': 10001,
    'DF_SENSOR_ACCELERATION_Y': 10002,
    'DF_SENSOR_ACCELERATION_Z': 10003,
    'DF_SENSOR_DISPLACEMENT_PEAK_INFO': 9,
    # I assign X100 for Async data
    'DF_SENSOR_DISPLACEMENT_PEAK_ASYNC_MAX': 9101,
    # I assign X100 for Async data
    'DF_SENSOR_DISPLACEMENT_PEAK_ASYNC_MIN': 9102,
    # I assign X100 for Async data
    'DF_SENSOR_DISPLACEMENT_PEAK_ASYNC_CURR': 9103,
    'DF_SENSOR_DISPLACEMENT_PEAK_MAX': 9001,
    'DF_SENSOR_DISPLACEMENT_PEAK_MIN': 9002,
    'DF_SENSOR_DISPLACEMENT_PEAK_CURR': 9003,
    'DF_SENSOR_MOTION_DETECTION': 11,
    'DF_SENSOR_MOTION_DETECTION_ASYNC': 11101,
    'DF_ACTUATION': 12,
    'DF_ACTUATION_ASYNC': 12101,
    'DF_SENSOR_GENERIC_DIFF_INP': 13,
    'DF_SENSOR_GENERIC_DIFF_INP_VALUE': 13001,  # 13 * 1000 + 1
    'DF_SENSOR_GENERIC_DIFF_INP_IA_GAIN': 13002,  # 13 * 1000 + 2
    'DF_REPEATER_INFO': 14,
    'DF_REPEATER_CHILD_NUM': 14001,  # 14 * 1000 + 1
    'DF_REPEATER_SYNC_WITH_SENIMAX': 14002,  # 14 * 1000 + 2
    'DF_REPEATER_XMIT_PERIOD_SEC': 14003,  # 14 * 1000 + 3

    'DF_SENSOR_VIBRATION_ASYNC_XYZ': 15,
    # DF_SENSOR_VIBRATION_ASYNC_XYZ * 1000 + 1
    'DF_SENSOR_VIBRATION_ASYNC_X': 15001,
    # DF_SENSOR_VIBRATION_ASYNC_XYZ * 1000 + 2
    'DF_SENSOR_VIBRATION_ASYNC_Y': 15002,
    # DF_SENSOR_VIBRATION_ASYNC_XYZ * 1000 + 3
    'DF_SENSOR_VIBRATION_ASYNC_Z': 15003,

    'DF_SENSOR_VIBRATION_REGULAR_XYZ': 16,
    # DF_SENSOR_VIBRATION_REGULAR_XYZ * 1000 + 1
    'DF_SENSOR_VIBRATION_REGULAR_X': 16001,
    # DF_SENSOR_VIBRATION_REGULAR_XYZ * 1000 + 2
    'DF_SENSOR_VIBRATION_REGULAR_Y': 16002,
    # DF_SENSOR_VIBRATION_REGULAR_XYZ * 1000 + 3
    'DF_SENSOR_VIBRATION_REGULAR_Z': 16003,
    # DF_SENSOR_VIBRATION_REGULAR_X + 100
    'DF_SENSOR_VIBRATION_REGULAR_BASE_X': 16101,
    # DF_SENSOR_VIBRATION_REGULAR_Y + 100
    'DF_SENSOR_VIBRATION_REGULAR_BASE_Y': 16102,
    # DF_SENSOR_VIBRATION_REGULAR_Z + 100
    'DF_SENSOR_VIBRATION_REGULAR_BASE_Z': 16103,
    # DF_SENSOR_VIBRATION_REGULAR_XYZ * 1000 + 100 + 1
    'DF_SENSOR_VIBRATION_REGULAR_THRESHOLD': 16101,
    # DF_SENSOR_VIBRATION_REGULAR_XYZ * 1000 + 100 + 2
    'DF_SENSOR_VIBRATION_REGULAR_INTERVAL': 16102,


    'DF_SENSOR_VIBRATION_EVENT_XYZ': 17,
    # DF_SENSOR_VIBRATION_EVENT_XYZ * 1000 + 1
    'DF_SENSOR_VIBRATION_EVENT_X': 17001,
    # DF_SENSOR_VIBRATION_EVENT_XYZ * 1000 + 2
    'DF_SENSOR_VIBRATION_EVENT_Y': 17002,
    # DF_SENSOR_VIBRATION_EVENT_XYZ * 1000 + 3
    'DF_SENSOR_VIBRATION_EVENT_Z': 17003,
    'DF_SENSOR_VIBRATION_EVENT_BASE_X': 17101,  # DF_SENSOR_VIBRATION_EVENT_X + 100
    'DF_SENSOR_VIBRATION_EVENT_BASE_Y': 17102,  # DF_SENSOR_VIBRATION_EVENT_Y + 100
    'DF_SENSOR_VIBRATION_EVENT_BASE_Z': 17103,  # DF_SENSOR_VIBRATION_EVENT_Z + 100

    'DF_DISPLACEMENT_3D_INFO': 18,
    'DF_DISPLACEMENT_3D_INFO_S1': 18001,  # DF_DISPLACEMENT_3D_INFO * 1000 + 1
    'DF_DISPLACEMENT_3D_INFO_S2': 18002,  # DF_DISPLACEMENT_3D_INFO * 1000 + 2
    'DF_DISPLACEMENT_3D_INFO_S3': 18003,  # DF_DISPLACEMENT_3D_INFO * 1000 + 3
    'DF_DISPLACEMENT_3D_INFO_S4': 18004,  # DF_DISPLACEMENT_3D_INFO * 1000 + 4
    'DF_DISPLACEMENT_3D_INFO_X': 18005,  # DF_DISPLACEMENT_3D_INFO * 1000 + 5
    'DF_DISPLACEMENT_3D_INFO_Y': 18006,  # DF_DISPLACEMENT_3D_INFO * 1000 + 6
    'DF_DISPLACEMENT_3D_INFO_Z': 18007,  # DF_DISPLACEMENT_3D_INFO * 1000 + 7
    'DF_DISPLACEMENT_3D_INFO_ANGL1': 18008,  # DF_DISPLACEMENT_3D_INFO * 1000 + 8
    'DF_DISPLACEMENT_3D_INFO_ANGL2': 18009,  # DF_DISPLACEMENT_3D_INFO * 1000 + 9
    'DF_DISPLACEMENT_3D_INFO_ANGL3': 18010,  # DF_DISPLACEMENT_3D_INFO * 1000 + 10
    'DF_DISPLACEMENT_3D_INFO_ANGL4': 18011,  # DF_DISPLACEMENT_3D_INFO * 1000 + 11
    'DF_DISPLACEMENT_3D_INFO_XX': 18012,  # DF_DISPLACEMENT_3D_INFO * 1000 + 12

    'DF_SENIMAX_STATUS_INFO': 22,
    'DF_SENIMAX_STATUS_INFO_TEMP': 22001,  # DF_SENIMAX_STATUS_INFO * 1000 + 1
    'DF_SENIMAX_STATUS_INFO_VOLT': 22002,  # DF_SENIMAX_STATUS_INFO * 1000 + 2
    'DF_SENIMAX_STATUS_INFO_RSSI': 22003,  # DF_SENIMAX_STATUS_INFO * 1000 + 3
    'DF_SENIMAX_STATUS_INFO_VRSN': 22004,  # DF_SENIMAX_STATUS_INFO * 1000 + 4
    'DF_SENIMAX_STATUS_INFO_VOLT_AUX': 22005,  # DF_SENIMAX_STATUS_INFO * 1000 + 5
    # DF_SENIMAX_STATUS_INFO * 1000 + 6
    'DF_SENIMAX_STATUS_INFO_CHARGE_CURR_1': 22006,
    # DF_SENIMAX_STATUS_INFO * 1000 + 7
    'DF_SENIMAX_STATUS_INFO_CHARGE_CURR_2': 22007,

    'DF_SENIMAX_STATUS_INFO_TX_INTERVAL': 22008,  # DF_SENIMAX_STATUS_INFO * 1000 + 8

    'DF_SENSOR_E_TILT_2D_XYZ': 23,
    'DF_SENSOR_E_TILT_2D_Pitch': 23001,
    'DF_SENSOR_E_TILT_2D_Roll': 23002,

    'DF_SENSOR_FORCE_BALANCE_VIBRATION_X': 24,
    'DF_SENSOR_FORCE_BALANCE_VIBRATION_Y': 25,
    'DF_SENSOR_FORCE_BALANCE_VIBRATION_Z': 26,

    # DF_SENSOR_FORCE_BALANCE_VIBRATION_X * 1000 + 100 + 1
    'DF_SENSOR_FORCE_BALANCE_X_AVERAGE_TIMEOFFSET': 24101,
    # DF_SENSOR_FORCE_BALANCE_VIBRATION_X * 1000 + 100 + 2
    'DF_SENSOR_FORCE_BALANCE_X_THRESHOLD_INTERVAL': 24102,
    # DF_SENSOR_FORCE_BALANCE_VIBRATION_Y * 1000 + 100 + 1
    'DF_SENSOR_FORCE_BALANCE_Y_AVERAGE_TIMEOFFSET': 25101,
    # DF_SENSOR_FORCE_BALANCE_VIBRATION_Y * 1000 + 100 + 2
    'DF_SENSOR_FORCE_BALANCE_Y_THRESHOLD_INTERVAL': 25102,
    # DF_SENSOR_FORCE_BALANCE_VIBRATION_Z * 1000 + 100 + 1
    'DF_SENSOR_FORCE_BALANCE_Z_AVERAGE_TIMEOFFSET': 26101,
    # DF_SENSOR_FORCE_BALANCE_VIBRATION_Z * 1000 + 100 + 2
    'DF_SENSOR_FORCE_BALANCE_Z_THRESHOLD_INTERVAL': 26102,

    'DF_SENSOR_HIGH_RATE_STRAIN': 27,

    'DF_SENSOR_GENERIC_SAMPLES': 28,

    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_1D * 100 + 1
    'DF_SENSOR_WIND_SPEED_1D': 28301,
    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_1D * 100 + 2
    'DF_SENSOR_WIND_DIRECTION_1D': 28302,

    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_3D * 100 + 0
    'DF_SENSOR_WIND_SPEED_3D': 28500,
    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_3D * 100 + 1
    'DF_SENSOR_WIND_AZIMUTH': 28501,
    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_3D * 100 + 2
    'DF_SENSOR_WIND_ELEVATION': 28502,
    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_3D * 100 + 3
    'DF_SENSORSOUND_SPEED': 28503,
    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_3D * 100 + 4
    'DF_SENSOR_AIR_TEMPERATURE': 28504,

    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_VIBRATION_X * 100 + 0
    'DF_SENSOR_VIBRATION_3D_X': 28000,
    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_VIBRATION_Y * 100 + 0
    'DF_SENSOR_VIBRATION_3D_Y': 28100,
    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_VIBRATION_Z * 100 + 0
    'DF_SENSOR_VIBRATION_3D_Z': 28200,

    # DF_SENSOR_PADDING * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_1D * 100 + 1
    'DF_SENSOR_WIND_SPEED_1D_NOTIF': 15101,
    # DF_SENSOR_PADDING * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_1D * 100 + 2
    'DF_SENSOR_WIND_DIRECTION_1D_NOTIF': 15102,

    # DF_SENSOR_PADDING * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_3D * 100 + 0
    'DF_SENSOR_WIND_SPEED_3D_NOTIF': 15500,
    # DF_SENSOR_PADDING * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_3D * 100 + 1
    'DF_SENSOR_WIND_AZIMUTH_NOTIF': 15501,
    # DF_SENSOR_PADDING * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_3D * 100 + 2
    'DF_SENSOR_WIND_ELEVATION_NOTIF': 15502,
    # DF_SENSOR_PADDING * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_3D * 100 + 3
    'DF_SENSORSOUND_SPEED_NOTIF': 15503,
    # DF_SENSOR_PADDING * 1000 + DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_3D * 100 + 4
    'DF_SENSOR_AIR_TEMPERATURE_NOTIF': 15504,

    # DF_SENSOR_PADDING * 1000 + DF_SENSOR_GENERIC_SUBTYPE_VIBRATION_X * 100 + 0
    'DF_SENSOR_VIBRATION_3D_X_NOTIF': 15000,
    # DF_SENSOR_PADDING * 1000 + DF_SENSOR_GENERIC_SUBTYPE_VIBRATION_Y * 100 + 0
    'DF_SENSOR_VIBRATION_3D_Y_NOTIF': 15100,
    # DF_SENSOR_PADDING * 1000 + DF_SENSOR_GENERIC_SUBTYPE_VIBRATION_Z * 100 + 0
    'DF_SENSOR_VIBRATION_3D_Z_NOTIF': 15200,

    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_LR_REDUNDANT_STRAIN * 100 + 0
    'DF_SENSOR_LR_REDUNDANT_STRAIN_DIFFRENTIAL': 28600,
    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_LR_REDUNDANT_STRAIN * 100 + 1
    'DF_SENSOR_LR_REDUNDANT_STRAIN_POSITIVE': 28601,
    # DF_SENSOR_GENERIC_SAMPLES * 1000 + DF_SENSOR_GENERIC_SUBTYPE_LR_REDUNDANT_STRAIN * 100 + 2
    'DF_SENSOR_LR_REDUNDANT_STRAIN_NEGATIVE': 28602,


    'MD_DEVICE_TYPE_SENSOR': 1,
    'MD_DEVICE_TYPE_ACTUATOR': 2,
    'ACTUATION_STARTED_ASYNC': 2,
    'ACTUATION_ENDED_ASYNC': 1,

    'SENSOR_DISPLACEMENT_IS_A_4_inch': 100,
    'SENSOR_DISPLACEMENT_IS_A_2_inch': 50,
    'SENSOR_DISPLACEMENT_IS_A_1_inch': 25,  # Not implemented for all sensors



    'PARSER_MODE_LIVE': 0,
    'PARSER_MODE_OFFLINE': 1,



    'DF_SENSOR_GENERIC_SUBTYPE_VIBRATION_X': 0,
    'DF_SENSOR_GENERIC_SUBTYPE_VIBRATION_Y': 1,
    'DF_SENSOR_GENERIC_SUBTYPE_VIBRATION_Z': 2,
    'DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_1D': 3,
    'DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_2D': 4,
    'DF_SENSOR_GENERIC_SUBTYPE_ANEMOMETER_3D': 5,

    'DF_SENSOR_GENERIC_SUBTYPE_LR_REDUNDANT_STRAIN': 6,

    'DF_INTERNAL_TEMP_FLIP_THRESHOLD': 175
}
# ------------------------Table: RegInfo.registration ---------------------
RegInfo_registration_columns = [
    "idRegistration",
    "DID",
    "DNAME",
    "DTYPE",
    "SID",
    "SNAME",
    "GROUPID",
    "M",
    "DATAFORMAT",
    "QUANTITYNAME",
    "UNIT",
    "NUMBERFORMAT",
    "NUMPRECISION",
    "DATETIMEFORMAT",
    "CALIBCOEF",
    "CALIBDOFF",
    "TEMPCALIBCOEF0",
    "TEMPCALIBCOEF1",
    "TEMPCALIBCOEF2",
    "TEMPCALIBCOEF3",
    "APPLYOPTIONALOFFSET",
    "SHOWABLE",
    "VERSION",
    "ALTUNIT",
    "AUCOEF",
    "AUDOFF"
]
RegInfo_registration_Index = {
    'DID': 1,
    'SID': 4,
    'SNAME': 5,
    'DataFormat': 8,
    'DTYPE': 28,
    'QuantityName': 9,
    'Unit': 10,
    'NumPrecision': 12,
    'CalibCOEF': 14,
    'CalibDOFF': 15,
}

# -----------------------Table: ActiveUsers Column Definition------------
ActiveUserIndex = {
    'Username': 1,
    'DBPass': 6,
    'AlertActivated': 4}
# ------------------------Table: Raw Data Table Definition------------
RawDataTableIndex = {
    'Time': 0,
    # 'SeqNum': 1,
    'Data': 1
    # 'Optional': 3
}
# -----------------------Table: Alert1D Column Definition------------
Alert1ColumnNumber = 46
Alert1DIndex = {
    'id': 0,
    'Status': 2,
    'AlertInterval': 3,
    'DID1': 4,
    'SID1': 5,
    'DATAFORMAT1': 6,
    'TempCoef': 10,
    'QUANTITYNAME1': 44,
    'W_L': 18,
    'W_H': 22,
    'A_L': 26,
    'A_H': 30,
    'Email1': 37,
    'Email2': 38,
    'Email3': 39,
    'Activated': 34,
    'LastModifDateTime': 35
}
# ---------------------Quantity List Column Definition -------------
QLColumnNumber = 24
QLIndex = {
    'DID': 1,
    'DNAME': 2,
    'SID': 3,
    'SNAME': 4,
    'DTYPE': 5,
    'DataFormat': 6,
    'QuantityName': 7,
    'Unit': 8,
    'NumPrecision': 10,
    'CalibCOEF': 11,
    'CalibDOFF': 12,
}
# ---------------------Temperature Calib Column Definition -------------
TCIndex = {
    'idTempCalib': 0,
    'DID': 1,
    'SID': 2,
    'DataFormat': 3,
    'Coef': 5
}
# -----------------------Table: Alert2D Column Definition------------
Alert2DIndex = {
    'Status': 0,
    'DID1': 1,
    'DNAME1': 2,
    'SID1': 3,
    'SNAME1': 4,
    'DATAFORMAT1': 5,
    'QUANTITYNAME1': 6,
    'COEF1': 7,
    'DOFF1': 8,
    'TEMPCOEF1': 9,
    'DID2': 10,
    'DNAME2': 11,
    'SID2': 12,
    'SNAME2': 13,
    'DATAFORMAT2': 14,
    'QUANTITYNAME2': 15,
    'COEF2': 16,
    'DOFF2': 17,
    'TEMPCOEF2': 18,
    'W1X': 19,
    'W1Y': 20,
    'W2X': 21,
    'W2Y': 22,
    'W3X': 23,
    'W3Y': 24,
    'W4X': 25,
    'W4Y': 26,
    'A1X': 27,
    'A1Y': 28,
    'A2X': 29,
    'A2Y': 30,
    'A3X': 31,
    'A3Y': 32,
    'A4X': 33,
    'A4Y': 34,
    'AlertInterval': 35,
    'Email1': 36,
    'Email2': 37,
    'Email3': 38,
    'Activated': 39,
    'id': 40
}
# ---------------Alarm Status---------------
AlertStatus = {
    'Normal': 0,
    'Warning': 1,
    'Alarm': 2,
    'Offline': 5
}

AlertStatusRev = {
    0: 'Normal',
    1: 'Warning',
    2: 'Alarm',
    5: 'Offline'
}

AlertStatusColorRev = {
    0: 'green',
    1: '#ffa31a',
    2: 'red'
}

AlertSubject = {
    0: 'Clear: ',
    1: 'Warning: ',
    2: 'Alarm: ',
    5: 'Offline: '
}

AlertSign = {
    0: ' =/= ',
    1: ' > ',
    2: ' >= ',
    3: ' <= ',
    4: ' < ',
}

MYSQL_Datetime_Formatter = "%Y-%m-%d %H:%M:%S"

def generate_data_query(deviceID, siteID, dataFormat, datetimeStart, datetimeEnd, coef, offset, temp_calib=[0,0,0,0], apply_temp_coef='False'):
    query = ""
    datetimePattern = re.compile(r'^[0-9]{4}-[0-9]{1,2}-[0-9]{1,2} [0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}')
    if datetimePattern.match(datetimeStart):
        datetimeStart=f"\'{datetimeStart}\'"
    if datetimePattern.match(datetimeEnd):
        datetimeEnd=f"\'{datetimeEnd}\'"
    # ------Assemble the Raw Data Table name and remove the "-" in the DID and SID-----
    tableNamePrefix = ("`Data_" + siteID + "`.`Data."
                        + deviceID + "."
                        + siteID + ".").replace("-", "")
    tableNameTemp = tableNamePrefix + \
        str(SenScopeConfiguration['DF_SENSOR_TEMP']) + "`"
    tableName1 = tableNamePrefix + str(dataFormat) + "`"
    # -----Date Time Picker----------
    # -------Query-------------------
    if (apply_temp_coef == 'True'):
        # query = (
        #     f"SELECT UNIX_TIMESTAMP(CONVERT_TZ(t1.Time, '+00:00', @@global.time_zone)) as Time, "
        #     f"({str(coef)})*" #Calib Coef
        #     f"(t1.Value" #Raw
        #     f"-({str(offset)})" #Calib Offset
        #     f"-({temp_calib[0]})" # Constant Temp Offset
        #     f"-({temp_calib[1]})*(IF(t2.value<={SenScopeConfiguration['DF_INTERNAL_TEMP_FLIP_THRESHOLD']},t2.value,(t2.value-256)))" #1st order Temp Calib
        #     # f"-({temp_calib[2]})*(POWER((IF(t2.value<={SenScopeConfiguration['DF_INTERNAL_TEMP_FLIP_THRESHOLD']},t2.value,(t2.value-256))),2))" #2nd order Temp Calib
        #     # f"-({temp_calib[3]})*(POWER((IF(t2.value<={SenScopeConfiguration['DF_INTERNAL_TEMP_FLIP_THRESHOLD']},t2.value,(t2.value-256))),3))" #3rd order Temp Calib
        #     f" as Calibrated "
        #     f"FROM {tableName1} as t1 LEFT JOIN {tableNameTemp} as t2 on t1.Time=t2.Time "
        #     f"WHERE t1.Time Between {datetimeStart} and {datetimeEnd} "
        #     f"order by UNIX_TIMESTAMP(t1.Time) asc;"
        if (int(dataFormat) == SenScopeConfiguration['DF_SENSOR_TEMP']):
            query = (
                f"SELECT UNIX_TIMESTAMP(CONVERT_TZ(t1.Time, '+00:00', @@global.time_zone)) as Time, IF(t1.Value<={SenScopeConfiguration['DF_INTERNAL_TEMP_FLIP_THRESHOLD']},t1.value,(t1.value-256)) as Temp "
                f"FROM {tableName1} as t1 WHERE Time Between {datetimeStart} and {datetimeEnd} "
                f"order by UNIX_TIMESTAMP(t1.Time) asc;"
            )
        else:
            query = (
                f"SELECT UNIX_TIMESTAMP(CONVERT_TZ(t1.Time, '+00:00', @@global.time_zone)) as Time, "
                f"t1.Value as Temp" #Raw
                f"FROM {tableName1} as t1 WHERE Time Between {datetimeStart} and {datetimeEnd} "
                f"order by UNIX_TIMESTAMP(t1.Time) asc;"
            )
    elif (int(dataFormat)//1000 == SenScopeConfiguration['DF_SENSOR_VIBRATION_EVENT_XYZ'] or
        int(dataFormat)//1000 == SenScopeConfiguration['DF_SENSOR_VIBRATION_REGULAR_XYZ'] or
        int(dataFormat)//1000 >= SenScopeConfiguration['DF_SENSOR_GENERIC_SAMPLES'] or
        int(dataFormat) == SenScopeConfiguration['DF_SENSOR_FORCE_BALANCE_VIBRATION_X'] or
        int(dataFormat) == SenScopeConfiguration['DF_SENSOR_FORCE_BALANCE_VIBRATION_X'] or
        int(dataFormat) == SenScopeConfiguration['DF_SENSOR_FORCE_BALANCE_VIBRATION_X'] or
        int(dataFormat) == SenScopeConfiguration['DF_SENSOR_HIGH_RATE_STRAIN']):
        query = (
            f"SELECT UNIX_TIMESTAMP(CONVERT_TZ(SUBTIME(t1.Time, Optional/1000), '+00:00', @@global.time_zone)) as Time, ({str(coef)})*(t1.Value-({str(offset)})) as Calibrated "
            f"FROM {tableName1} as t1 WHERE SUBTIME(t1.Time, (Optional div 1000)) Between {datetimeStart} and {datetimeEnd} "
            f"order by (UNIX_TIMESTAMP(t1.Time)*1000 - Optional) asc;"
        )
    elif (int(dataFormat) == SenScopeConfiguration['DF_SENSOR_TEMP']):
        query = (
            f"SELECT UNIX_TIMESTAMP(CONVERT_TZ(t1.Time, '+00:00', @@global.time_zone)) as Time, ({str(coef)})*((IF(t1.Value<={SenScopeConfiguration['DF_INTERNAL_TEMP_FLIP_THRESHOLD']},t1.value,(t1.value-256)))-({str(offset)})) as Calibrated "
            f"FROM {tableName1} as t1 WHERE Time Between {datetimeStart} and {datetimeEnd} "
            f"order by UNIX_TIMESTAMP(t1.Time) asc;"
        )
    else:
        query = (
            f"SELECT UNIX_TIMESTAMP(CONVERT_TZ(t1.Time, '+00:00', @@global.time_zone)) as Time, ({str(coef)})*(t1.Value-({str(offset)})) as Calibrated "
            f"FROM {tableName1} as t1 WHERE t1.Time Between {datetimeStart} and {datetimeEnd} "
            f"order by UNIX_TIMESTAMP(t1.Time) asc;"
        )
    return query

def print_debug_info(input_str):
    print(f"======================================\n"
          f"{input_str}\n"
          f"======================================\n")
# ---------------------Fxn: read data to dataframe with mysqlconnector lib --------------------------------------
# dbCursor: mysqlconnector database cursor object
# query: MySQL query in string
# return: pandas.DataFrame object
def read_cursor_to_DF_mysqlconnector(dbCursor, query):
    try:
        dbCursor.execute(query)
        data = dbCursor.fetchall()
        frame = pd.DataFrame(list(data))
        frame.columns = list(dbCursor.column_names)
    except Exception as e:
        frame = pd.DataFrame()
        print("========\n", str(e), "\n========\nQuery:\n",query,"\n")
    return frame

def read_cursor_to_DF_mysqlclient(dbCursor, query, add_column_name=0):
    try:
        dbCursor.execute(query)
        data = dbCursor.fetchall()
        frame = pd.DataFrame(list(data))
        # print(dbCursor.description)
        if add_column_name == 1:
            column_list = list()
            for column_name in dbCursor.description:
                column_list.append(list(column_name)[0])
            frame.columns = list(column_list)
        # End if add_column_name
    except Exception as e:
        frame = pd.DataFrame()
        print("========\n", str(e), "\n========\nQuery:\n",query,"\n")
    return frame
# ---------------------Fxn: Time to now --------------------------------------
# timestamp: time read from mysql table, already Datetime.Datetime format
# return: seconds

def utc_time_to_now_timedelta(timeStamp):
    if timeStamp is not None:
        # dateTimeIn = datetime.strptime(timeStamp,"%Y-%m-%d %H:%M:%S")
        dateTimeNow = datetime.utcnow()
        time_delta = dateTimeNow - timeStamp
    else:
        time_delta = timedelta(days=7000)
    return time_delta



import mysql.connector
import numpy as np
import pandas as pd
import math

# 1. Establish a remote connection to resensys.net using python environment
# 2. Encrypt the connection to resensys.net
# 3. Pull general data for all sensors/sites/routing parameters for a given account

class Resensys():

    def __init__(self,username,password):
        self.username = username
        self.password = password
        self.host = 'resensys.net'
        self.connect()
    
    def connect(self):
        # define ssh connection

        # decrypt user password
        self.password_encoding_generator()
        config = {  
            'user': self.username,
            'password': self.encoded_password,
            'host': self.host,
            'port': 3306,

            # Create SSL connection with the self-signed certificate
            'ssl_verify_cert': False,
        }

        # self.conn = mysql.connector.connect(user=self.username,password=self.encoded_password,host=self.host)
        self.conn = mysql.connector.connect(**config,use_pure=True)
        return self.conn
    
    def getCredentials(self):
        return self.username,self.password
    
    def password_encoding_generator(self):
        password_bytes = bytes(self.password, 'ascii')
        byte_list = []
        for byte in password_bytes:
            byte_list.append(byte)
        reverse_list = byte_list[::-1]
        # loop through every element of the reversed array
        for i in range(len(reverse_list)):
            new_byte = (int(math.pow(-1, i))*(i%10))
            reverse_list[i] += new_byte
        # decode the new array
        encoded_characters = []
        for j in range(len(reverse_list)):
            encoded_characters.append(chr(reverse_list[j]))
        self.encoded_password = ''.join(encoded_characters)

    def add_new_site(self,SID):
        self.serverTool.add_site_to_user(self, self.username, SID, acc_pswd="", mode=0, extra_criterion="")


class Sensors(Resensys):
    def __init__(self,parent):
        super().__init__(parent.username, parent.password)
        self.conn = parent.conn

    # public
    def getSensors(self,format="json",filter={}):
        # create a db cursor
        cursor = self.conn.cursor()
        # define the query to the db
        query = f"SELECT TTL,DID,SID,TYP,DES,STT FROM `{self.username}`.`Info`"
        cursor.execute(query)
        results = cursor.fetchall()
        # define resultant columns as list
        cols = ["Title","DID","SID","DeviceType","Description","Status"]

        if format == "json":
            devices_dict = {}
            for count,result in enumerate(results):
                devices_dict[str(count)] = {cols[i]:result[i] for i in range(len(cols))}
            return devices_dict
        elif format == "dataframe":
            return pd.DataFrame(results,columns=cols)

    # public 
    def getSites(self,format="json",filter={},unique=False):
        # create a db cursor
        cursor = self.conn.cursor()
        # define the query to the db
        query = f"SELECT SIDName,SID FROM `{self.username}`.`Accounts`"
        cursor.execute(query)
        results = cursor.fetchall()
        # define resultant columns as list
        cols = ["SiteName","SID"]

        if unique:
            return pd.DataFrame(results,columns=cols).SID.unique()
        else:
            if format == "json":
                devices_dict = {}
                for count,result in enumerate(results):
                    devices_dict[str(count)] = {cols[i]:result[i] for i in range(len(cols))}
                return devices_dict
            elif format == "dataframe":
                return pd.DataFrame(results,columns=cols)

    def getTables(self,SID):
        # convert the SID XX-XX to xxxx
        sid = SID.replace('-','') 
        cursor = self.conn.cursor()
        query = ("SHOW TABLES FROM `Data_%s`"%(sid))
        cursor.execute(query)
        tables = cursor.fetchall()
        field_names = [i[0] for i in cursor.description]
        tables_df = pd.DataFrame(tables,columns=field_names)
        return tables_df

    # public
    def getRouting(self,format="json",filter={}):
        # get list of all sites
        sites = self.getSites(format="dataframe").SID.tolist()

        # create a bulk list of all tables in user sites
        combined_tables = []
        # loop through every site in sites
        for site in sites:
            site_tables = self.getTables(site)['Tables_in_Data_'+site.replace("-","")].tolist()
            combined_tables += site_tables

        volt_tables=[]
        # remove all but the volt tables
        for table in combined_tables:
            # get the dataformat
            df = table.split('.')[-1]
            did = table.split('.')[1][:4]
            sid = table.split('.')[2]
            # check if the data format is volt = 3003
            if df == '3003' and did != '0000':
                device_id = table.split('.')[1]
                device_id = '-'.join([device_id[i:i+2] for i in range(0, len(device_id), 2)])
                volt_tables.append((table,device_id,sid))
            
        # create a db cursor
        cursor = self.conn.cursor()
        local_addresses = []
        # define the query to the db
        for table in volt_tables:
            SID = table[2]
            query = ("SELECT `Optional` FROM `Data_%s`.`%s` ORDER BY Time DESC LIMIT 1"%(SID,table[0]))
            cursor.execute(query)
            optional = cursor.fetchall()
            local_address = int(optional[0][0]/1001)
            local_addresses.append((table[1],SID[:2]+"-"+SID[2:],local_address))
        
        la_df = pd.DataFrame(local_addresses,columns=['DID','SID','LocAddr'])
        return la_df


# 1. Handles report generation tasks
# 2. Used to define a scheduler for sending email/text notifications (every day, week, month, year, etc.)
# 3. Used to configure the content of reports
# 4. Handles the plotting and data visualization tasks using matplotlib.pyplot
# 5. Stores recipient information and report naming (e.g. name a report based on site)


from setuptools import setup, find_packages
import codecs
import os

VERSION = '1.3.0'
DESCRIPTION = 'Python package for managing and interacting with Resensys SenSpots.'
LONG_DESCRIPTION = """
<img src="https://resensys.net:8443/static/images/resensys-logo-hollow.bmp" alt="resensys" width="300"/>

# SenStream (v 1.3.0)

**SenStream** is a python package for managing and interacting with Resensys SenSpot devices. Streamline data flows from SenSpot technologies and high-level data pipelines with this high-level software development kit.

At a high level, the SenStream project is designed to bridge the gap between the structural/civil engineer and the software developer for **structural health monitoring** and other scientific applications. This package comes equipped with the following features for SenSpot management, custom time streams, and state-of-the art time series machine learning tools:

#### API Functionality

*   Extract and configure SenSpot parameters.
*   Pull time streams from SenSpots from the cloud

<br>
<img src="https://www.resensys.com/images/load-rating-app-note.png" alt="resensys_arch" width="700"/>

## Installation

#### Requirements

We recommend that you have a version of python >= 3.7 installed on your machine before proceeding. If the below `pip` command does not automatically install `pandas` and `numpy`, you should make sure that these libraries have been installed before proceeding.

#### Install using pip

To start using SenStream install using pip:

    pip install senstream

`pip` is the de facto and recommended package-management system written in Python and is used to install and manage software packages. It connects to an online repository of public packages, called the [Python Package Index](https://pypi.org/).

Note the `pip` refers to the Python 3 package manager. In environment where Python 2 is also available the correct command may be `pip3`.

## Getting Started

The following demonstrates what you need to get started with a connection to the Resensys cloud and check that you can extract data.



```
# import the necessary libraries and modules
import numpy as np
import pandas as pd

from senstream.resensys import Resensys,Sensors
from senstream.senspot import SenSpot

# for users with SenScope license, specify username and password
username,password = "test_user","user_password" 

# create a client connection to resensys.net
client = Resensys(username,password)

# create Sensors object to check all sensors in present account
sensors = Sensors(client)

# get list of all devices in the client account as pandas dataframe
sensor_df = sensors.getSensors(format="dataframe")

# print list of all the sensors in the account
print(sensor_df['DID'].tolist())
```

### View all active site registered for current user:
```sensors.getSites(format="dataframe")```
### View current routing parameters for SenSpots:
```sensors.getRouting(format="dataframe").head(5)  # display first five records```
## SenSpot Object
Create an instance of a SenSpot object by referencing the previous client connection and specifying the device ID of the SenSpot from the user's sensor list.

```senspot = SenSpot(client,"15-03-27-74")```

### Get the name assigned to the SenSpot:
```senspot.getName()```
Returns **string**.

### Get the site ID assignment for the SenSpot:
```senspot.getSite()```
Returns **string** of format "xx-xx" with site ID.

### Get the local address assigned to the SenSpot (useful for wireless routing):
```senspot.getLocAddr()```
Returns **int**.

### View all available quantities associated with SenSpot:
```senspot.getQuantities()```
Returns **[string]**

### Check current calibration coefficients for SenSpot:
```senspot.getCoefficients()```
Returns **dict**

### Get general device summary of SenSpot:
```senspot.getDeviceInfo()```
Returns **dict**

## Extract Time Series from SenSpot
* **df_name**: quantity being pushed by the SenSpot (complete list given by calling **getQuantities()** method)
* **time**: options = "1hour","2hour","6hour","12hour","24hour","48hour","1week","2week","4week","6month","12month","2year","custom"

If you choose a custom time range, then you must specify additional parameters:
* **t_s**: Start Time (UTC) (e.g. "2020-09-21 17:00:00")
* **t_e**: End Time (UTC) (e.g. "2020-09-21 17:00:00")

### Example to pull previous two weeks of data for SenSpot quantities "Strain-xx-high_rate" and "Internal Temperature"
```ss_strain = senspot.timeStream("Strain-xx-high_rate",time=["2week"])```

```ss_temperature = senspot.timeStream("Internal Temperature",time=["2week"])```

Returns **pandas.DataFrame** object.


## Support

For any question on the usage of PROJECT please use the [Resensys Community Portal](). If you found a problem with the software, please [create an issue](https://github.com/resensys/PROJECT/issues) on GitHub. If you are a Greenbone customer, you may alternatively or additionally forward your issue to the Resensys Support Portal.

## Maintainer

This project is maintained by [Tom Wade](https://www.linkedin.com/in/thomas-shane-wade/) at Resensys, LLC.

## Contributing

Your contributions are highly appreciated. Please [create a pull request](https://github.com/greenbone/PROJECT/pulls) on GitHub. Bigger changes need to be discussed with the development team via the [issues section at GitHub](https://github.com/greenbone/PROJECT/issues) first.

State here if contributions are welcome. State the requirements a contribution should meet to get merged.

Details about development, like creating a dev environment or running tests, also belong here, for example:

For development, you should use [pipenv](https://pipenv.readthedocs.io/en/latest/) to keep your Python packages separated in different environments. First install pipenv via pip

    pip install --user pipenv

Afterwards run

    pipenv install --dev

in the checkout directory of PROJECT (the directory containing the Pipfile) to install all dependencies including the packages only required for development.

If there are more specific suggestions for development or guidelines for contributions, consider sending an email to thomas.wade@resensys.com.


## License

Copyright (C) 2023 [Resensys, LLC.](https://www.resensys.com/index.html)

Licensed under the [GNU General Public License v3.0 or later](LICENSE).
"""

# Setting up
setup(
    name="senstream",
    version=VERSION,
    author="Resensys (Tom Wade)",
    author_email="<thomas.wade@resensys.com>",
    description='Python package for managing and interacting with Resensys SenSpots.',
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['mysql-connector-python', 'numpy', 'pandas', 'datetime','requests'],
    keywords=['python', 'stream', 'senspot', 'resensys', 'time series'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)

from setuptools import setup

setup(
    name="senstreamlib",
    version="1.0",
    author="Tom Wade",
    author_email="thomas.wade@resensys.com",
    description="Resensys Python SDK for building SenSpot Data Pipelines and Time Series Analysis Tools.",
    packages=['senstream'],
    python_reqires=">=3.7",
)

# *   Push custom time streams to the cloud with `stream_builder` module
# *   Assign and configure custom email/SMS alerts
# *   Setup custom and automated report generation
# *   Stream real-time data in laboratory environment with `lab_streamer` module
# *   Time-series analysis/model training (regression, classification, clustering, anomaly detection, etc.)


import mysql.connector
from senstream.resensys import Resensys
from datetime import datetime,timedelta
import pandas as pd

# 1. Handles checking senspot credentials, info, calibrations
# 2. Handles checking the routing parameters
# 3. Contains some db helper function for handling dataformat to format name translation
# 4. timestream: Used to pull time series for a single SenSpot for different time periods
# 5. Sync calibration coefficients across systems (senscope and webportal); includes temperature compensation

class SenSpot(Resensys):

    def __init__(self,parent,did):
        super().__init__(parent.username, parent.password)
        self.conn = parent.conn
        self.did = did
    
    # public
    def getName(self):
        # create a db cursor
        cursor = self.conn.cursor()
        # define the query to the db
        query = f"SELECT TTL FROM `{self.username}`.`Info` WHERE `DID`='{self.did}'"
        cursor.execute(query)
        site = cursor.fetchall()[0][0]
        return site

    # public
    def getSite(self):
        # create a db cursor
        cursor = self.conn.cursor()
        # define the query to the db
        query = f"SELECT SID FROM `{self.username}`.`Info` WHERE `DID`='{self.did}'"
        cursor.execute(query)
        site = cursor.fetchall()[0][0]
        return site
    
    # public
    def getLocAddr(self):
        sid = self.getSite().replace("-","")
        table = "Data."+self.did.replace("-","")+"."+sid+".3003"
        # create a db cursor
        cursor = self.conn.cursor()
        # define the query to the db
        query = f"SELECT Optional FROM `Data_{sid}`.`{table}` ORDER BY Time DESC LIMIT 1"
        cursor.execute(query)
        la = int(cursor.fetchall()[0][0]/1001)
        return la
    
    # public
    def getQuantities(self,translate=True):
        cursor = self.conn.cursor()
        df_query = ("""SELECT TRIM(LEADING '0' FROM M1) AS M1,
                        TRIM(LEADING '0' FROM M2) AS M2,
                        TRIM(LEADING '0' FROM M3) AS M3,
                        TRIM(LEADING '0' FROM M4) AS M4,
                        TRIM(LEADING '0' FROM M5) AS M5,
                        TRIM(LEADING '0' FROM M6) AS M6,
                        TRIM(LEADING '0' FROM M7) AS M7,
                        TRIM(LEADING '0' FROM M8) AS M8,
                        TRIM(LEADING '0' FROM M9) AS M9,
                        TRIM(LEADING '0' FROM M10) AS M10
                        FROM `%s`.Info WHERE DID = '%s'""" % (self.username,self.did))
        cursor.execute(df_query)
        dataformats = cursor.fetchall()

        non_empty = []
        for i in range(len(dataformats[0])):
            if dataformats[0][i] != '':
                non_empty.append(dataformats[0][i])

        if translate:
            non_empty = [self.get_quantity_name(non_empty[i]) for i in range(len(non_empty))]
        return non_empty

    def get_quantity_name(self,dataformat):
        cnx = mysql.connector.connect(user='regUser',password='iZge8097^ds6',
                            host='resensys.net',use_pure=True)
        c = cnx.cursor()
        query = ("SELECT Name FROM `RegInfo`.`quantityBank` WHERE `DataFormat` ='%s'"%(dataformat))
        c.execute(query)
        quantity_name = c.fetchall()
        cnx.close()
        return quantity_name[0][0]

    def getDataFormat(self,df_name):
        cnx = mysql.connector.connect(user='regUser',password='iZge8097^ds6',
                            host='resensys.net',use_pure=True)
        c = cnx.cursor()
        query = ("SELECT DataFormat,Unit FROM `RegInfo`.`quantityBank` WHERE `Name` ='%s'"%(df_name))
        c.execute(query)
        quantity_name = c.fetchall()
        cnx.close()
        return quantity_name[0][0],quantity_name[0][1]

    def getCalibration(self,df_name=""):
        cursor = self.conn.cursor()
        df_query = ("""SELECT TRIM(LEADING '0' FROM M1) AS M1,
                        TRIM(LEADING '0' FROM M2) AS M2,
                        TRIM(LEADING '0' FROM M3) AS M3,
                        TRIM(LEADING '0' FROM M4) AS M4,
                        TRIM(LEADING '0' FROM M5) AS M5,
                        TRIM(LEADING '0' FROM M6) AS M6,
                        TRIM(LEADING '0' FROM M7) AS M7,
                        TRIM(LEADING '0' FROM M8) AS M8,
                        TRIM(LEADING '0' FROM M9) AS M9,
                        TRIM(LEADING '0' FROM M10) AS M10
                        FROM `%s`.`Info` WHERE DID = '%s'""" % (self.username,self.did))
        cursor.execute(df_query)
        dataformats = cursor.fetchall()[0]

        dataformat,unit = self.getDataFormat(df_name)

        index = 1
        for i in range(len(dataformats)):
            if dataformats[i] == dataformat:
                index = i+1

        coeff_col,offset_col = "COF"+str(index),"DOF"+str(index)
        coeff_query = f"SELECT `{coeff_col}`,`{offset_col}` FROM `{self.username}`.`Info` WHERE `DID` = '{self.did}'"
        cursor.execute(coeff_query)
        coeffs = cursor.fetchall()[0]

        return coeffs[0],coeffs[1],unit

    # public
    def getCoefficients(self):
        coefficients = {}
        quantities = self.getQuantities()
        for quantity in quantities:
            coeff,offset,unit = self.getCalibration(quantity)
            coefficients[str(quantity)] = {"coefficient":coeff,"offset":offset,"unit":unit}
        return coefficients

    # public
    def getDeviceInfo(self,properties=[]):
        device_info = {}
        if "Name" in properties:
            device_info["Name"] = self.getName()
        if "DID" in properties:
            device_info["DID"] = self.did
        if "SID" in properties:
            device_info["SID"] = self.getSite()
        if "LocAddr" in properties:
            device_info["LocAddr"] = self.getLocAddr()
        if "Quantities" in properties:
            device_info["Quantities"] = self.getCoefficients()
        if len(properties) == 0:
            device_info["Name"] = self.getName()
            device_info["DID"] = self.did
            device_info["SID"] = self.getSite()
            device_info["LocAddr"] = self.getLocAddr()
            device_info["Quantities"] = self.getCoefficients()
        return device_info

    # public
    def timeStream(self,df_name,time=[],t_s="",t_e="",calibrate=True):
        did = str(self.did).replace('-',"")
        sid = str(self.getSite()).replace('-',"")
        db = "Data_"+str(sid)
        DF,unit = self.getDataFormat(df_name)
        table = db+"."+"`Data."+did+"."+sid+"."+DF+"`"

        t_end = datetime.utcnow()
        t_start = t_end - timedelta(hours=24)

        # define options for time window
        if "1hour" in time:
            t_start = t_end - timedelta(hours=1)
        elif "2hour" in time:
            t_start = t_end - timedelta(hours=2)
        elif "6hour" in time:
            t_start = t_end - timedelta(hours=6)
        elif "12hour" in time:
            t_start = t_end - timedelta(hours=12)
        elif "24hour" in time or "1day" in time:
            t_start = t_end - timedelta(hours=24)
        elif "48hour" in time or "2day" in time:
            t_start = t_end - timedelta(hours=48)
        elif "1week" in time:
            t_start = t_end - timedelta(weeks=1)
        elif "2week" in time:
            t_start = t_end - timedelta(weeks=2)
        elif "4week" in time or "1month" in time:
            t_start = t_end - timedelta(weeks=4)
        elif "6month" in time:
            t_start = t_end - timedelta(months=6)
        elif "12month" in time or "1year" in time:
            t_start = t_end - timedelta(months=12)
        elif "2year" in time:
            t_start = t_end - timedelta(years=2)
        else:
            t_start = t_end - timedelta(hours=24)

        try:
            if "custom" in time:
                t_start,t_end = t_s,t_e
        except Exception as e:
            print("Please check that time = ['custom']")
        cursor = self.conn.cursor()

        query = ("SELECT * FROM %s WHERE (Time >= '%s' AND Time <= '%s')" % (table,t_start,t_end))
        # Execute SQL query
        cursor.execute(query)
        # Record all instances that match query to records
        records = cursor.fetchall()
        # Collect the name of each of the table fields
        field_names = [i[0] for i in cursor.description]
        # Convert the records to a dataframe and filter rows by start and end time
        data_df = pd.DataFrame(records,columns=field_names)
        
        # Set added time to a TimeDelta object
        data_df['Time Added'] = pd.to_timedelta(data_df['Optional'],'milli')
        data_df['Time'] = pd.to_datetime(data_df['Time'])
        # Subtract the retarded time from the base time
        data_df['Time'] = data_df['Time'] - data_df['Time Added']
        # Drop the temporary 'Time Added' feature from data_df
        data_df = data_df.drop('Time Added',axis=1)                   
        
        # apply flip threshold if internal temperature measurement
        if (DF == '3002'):
            for i in range(len(data_df)):
                temp = data_df.Value[i]
                if temp > 175:
                    data_df.Value[i] = temp-256

        # Apply coefficients to the filtered data
        if calibrate:
            coeff,offset,unit = self.getCalibration(df_name)
            data_df['Value'] = coeff*(data_df['Value']-offset)
        
        return data_df.iloc[:,[0,2]].drop_duplicates().reset_index(drop=True)
